@@ -76,55 +76,78 @@ A direct raw writing/reading to the communication port
     </pre>
 
 """
+from __future__ import print_function
 
-def getDeviceClassObjects(classname,path=''):
+
+def getDeviceClassObjects(classname, path=""):
     """
     This method locates a Tango Device Class in the python Path and returns its (Device,DeviceClass) objects
     """
     import imp
-    desc = imp.find_module(classname)
-    print('getDeviceClassObjects(%s) from %s'%(classname,desc))
-    module = imp.load_module(classname,*desc)
-    return getattr(module,classname),getattr(module,classname+'Class')
 
-def addAllClasses(obj,servername='',db=None,classes=None):
+    desc = imp.find_module(classname)
+    print("getDeviceClassObjects(%s) from %s" % (classname, desc))
+    module = imp.load_module(classname, *desc)
+    return getattr(module, classname), getattr(module, classname + "Class")
+
+
+def addAllClasses(obj, servername="", db=None, classes=None):
     """
     This method will add all classes listed in the database to the server object (if those classes are available in the pythonpath)
-    
+
     obj argument must be a PyTango.Util(['exec_name','instance_name']) object
     If servername or db are not specified will be taken from obj.instance()
-    
+
     Dependences can be added to pythonpath using sys.path.insert(0,'path/to/your/device')
     """
     if classes is None:
         instance = obj.instance()
-        if not db:db = instance.get_database()
-        if not servername: servername = instance.get_ds_name()
-        if ' ' in servername or '.py' in servername: servename = '/'.join(s.replace('.py','').strip() for s in servername.split())
+        if not db:
+            db = instance.get_database()
+        if not servername:
+            servername = instance.get_ds_name()
+        if " " in servername or ".py" in servername:
+            servename = "/".join(
+                s.replace(".py", "").strip() for s in servername.split()
+            )
         classes = db.get_server_class_list(servername)
     for c in classes:
-        dev,devclass = getDeviceClassObjects(c)
-        print('Adding %s,%s to %s'%(dev,devclass,obj))
-        obj.add_TgClass(devclass,dev,c)
+        dev, devclass = getDeviceClassObjects(c)
+        print("Adding %s,%s to %s" % (dev, devclass, obj))
+        obj.add_TgClass(devclass, dev, c)
     return obj
-        
 
-def updateChildClassDicts(Child,Parent,Exclude=[]):
-    #print 'Updating %s from %s'%(Child.__name__,Parent.__name__)
+
+def updateChildClassDicts(Child, Parent, Exclude=[]):
+    # print 'Updating %s from %s'%(Child.__name__,Parent.__name__)
     for attribute in dir(Child):
-        cattr = getattr(Child,attribute)
-        if (not attribute.startswith('__')) and isinstance(cattr,dict) and hasattr(Parent,attribute):
-            pattr = getattr(Parent,attribute)
-            if isinstance(pattr,dict): 
-                #print 'Updating %s.%s from %s'%(Child.__name__,attribute,Parent.__name__)
-                cattr.update((k,v) for k,v in pattr.items() if k not in Exclude and k not in cattr)
-#updateChildClassDicts(AlbaPLCClass,PyPLCClass)        
-        
-def NewTypeInheritance(name,klass,parent,dikt={}):
-    """ Based on NewClass = type('nameOfClass',(bases,),dikt={}) """
-    return type(name,(klass,parent),dikt)
+        cattr = getattr(Child, attribute)
+        if (
+            (not attribute.startswith("__"))
+            and isinstance(cattr, dict)
+            and hasattr(Parent, attribute)
+        ):
+            pattr = getattr(Parent, attribute)
+            if isinstance(pattr, dict):
+                # print 'Updating %s.%s from %s'%(Child.__name__,attribute,Parent.__name__)
+                cattr.update(
+                    (k, v)
+                    for k, v in list(pattr.items())
+                    if k not in Exclude and k not in cattr
+                )
 
-def FullTangoInheritance(name,child,childClass,parent,parentClass,Exclude=[],ForceDevImpl=False):
+
+# updateChildClassDicts(AlbaPLCClass,PyPLCClass)
+
+
+def NewTypeInheritance(name, klass, parent, dikt={}):
+    """Based on NewClass = type('nameOfClass',(bases,),dikt={})"""
+    return type(name, (klass, parent), dikt)
+
+
+def FullTangoInheritance(
+    name, child, childClass, parent, parentClass, Exclude=[], ForceDevImpl=False
+):
     """ arguments are: NewDeviceName, Device, DeviceClass, newParentDevice, newParentDeviceClass, parent members to exclude 
     @remark The code must be added always before each add_TgClass call (it could be outside the class definition file!)
     
@@ -139,61 +162,79 @@ def FullTangoInheritance(name,child,childClass,parent,parentClass,Exclude=[],For
     </pre>
     """
     if parent not in child.__bases__:
-        #THIS PRINT OUT BROKE FANDANGO.SH CALLS
-        #print 'Applying FullTangoInheritance from %s and %s' % (child, parent)
+        # THIS PRINT OUT BROKE FANDANGO.SH CALLS
+        # print 'Applying FullTangoInheritance from %s and %s' % (child, parent)
         if not ForceDevImpl:
-            newdevice = type(name,(child,parent),{})
-            newdeviceclass = type(name+'Class',(childClass,parentClass),{})
+            newdevice = type(name, (child, parent), {})
+            newdeviceclass = type(name + "Class", (childClass, parentClass), {})
         else:
             import PyTango
-            newdevice = type(name,(child,parent,PyTango.LatestDeviceImpl),{})
-            newdeviceclass = type(name+'Class',(childClass,parentClass,PyTango.DeviceClass),{})
-        updateChildClassDicts(newdeviceclass,parentClass,Exclude)
-        return newdevice,newdeviceclass
+
+            newdevice = type(name, (child, parent, PyTango.LatestDeviceImpl), {})
+            newdeviceclass = type(
+                name + "Class", (childClass, parentClass, PyTango.DeviceClass), {}
+            )
+        updateChildClassDicts(newdeviceclass, parentClass, Exclude)
+        return newdevice, newdeviceclass
     else:
-        return child,childClass
-    
+        return child, childClass
+
+
 def DeviceClassInheritance(ChildClass):
-    """ class decorator """
+    """class decorator"""
     for b in ChildClass.__bases__:
-        for p in ('class_property_list','device_property_list','cmd_list','attr_list'):
-            d = getattr(b,p,{})
-            d and getattr(ChildClass,p).update((k,v) for k,v in d.items() if k not in getattr(ChildClass,p))
+        for p in (
+            "class_property_list",
+            "device_property_list",
+            "cmd_list",
+            "attr_list",
+        ):
+            d = getattr(b, p, {})
+            d and getattr(ChildClass, p).update(
+                (k, v) for k, v in list(d.items()) if k not in getattr(ChildClass, p)
+            )
     return ChildClass
-    
-def addTangoInterfaces(device,interfaces):
-    """ It adds properties and implementation to the parent class from a list of tuples (Device,DeviceClass)
+
+
+def addTangoInterfaces(device, interfaces):
+    """It adds properties and implementation to the parent class from a list of tuples (Device,DeviceClass)
     @param device A tuple (Device,DeviceClass) to be extended with interfaces
     @param interfaces A list of tuples (Device,DeviceClass) to the first server
     @return Returns a tuple (Device,DeviceClass) containing the new interfaces
-    
+
         from interface import FullTangoInheritance
         PySignalSimulator,PySignalSimulatorClass = FullTangoInheritance('PySignalSimulator',PySignalSimulator,PySignalSimulatorClass,DynamicDS,DynamicDSClass,ForceDevImpl=True)
-        py.add_TgClass(PySignalSimulatorClass,PySignalSimulator,'PySignalSimulator')    
+        py.add_TgClass(PySignalSimulatorClass,PySignalSimulator,'PySignalSimulator')
     """
-    if any(type(p) is not tuple for p in ([device]+interfaces)):
-        raise Exception,'TangoInterface_ArgumentIsNotTuple'
-    device,deviceclass = device
-    for interface,interfaceclass in interfaces:
-        device,deviceclass = FullTangoInheritance(device.__name__, \
-            device,deviceclass, \
-            interface,interfaceclass, \
-            ForceDevImpl=True)
-    return device,deviceclass
-    
-def addCommandToTgClass(dev,dev_class,cmd_name,cmd_def,cmd_fun):
+    if any(type(p) is not tuple for p in ([device] + interfaces)):
+        raise Exception("TangoInterface_ArgumentIsNotTuple")
+    device, deviceclass = device
+    for interface, interfaceclass in interfaces:
+        device, deviceclass = FullTangoInheritance(
+            device.__name__,
+            device,
+            deviceclass,
+            interface,
+            interfaceclass,
+            ForceDevImpl=True,
+        )
+    return device, deviceclass
+
+
+def addCommandToTgClass(dev, dev_class, cmd_name, cmd_def, cmd_fun):
     """
     @param cmd_def should be like [[argintype,"desc"],[argouttype,"desc"],{'property':value}]
     e.g: [[PyTango.DevString, "formula to evaluate"],[PyTango.DevString, "formula to evaluate"],{'Display level':PyTango.DispLevel.EXPERT,}]
     """
-    #print 'Adding command %s to %s,%s device class' % (cmd_name,dev,dev_class)
-    setattr(dev,cmd_name,cmd_fun)
+    # print 'Adding command %s to %s,%s device class' % (cmd_name,dev,dev_class)
+    setattr(dev, cmd_name, cmd_fun)
     dev_class.cmd_list[cmd_name] = cmd_def
     return
 
-#def getAllBases(klass):
-    #""" Gets recursively all super classes for a given class """
-    #bases = list(klass.__bases__)
-    #for base in bases:
-        #[bases.append(b) for b in getAllBases(base) if b not in bases]
-    #return bases
+
+# def getAllBases(klass):
+# """ Gets recursively all super classes for a given class """
+# bases = list(klass.__bases__)
+# for base in bases:
+# [bases.append(b) for b in getAllBases(base) if b not in bases]
+# return bases
